/*
 *  Uzebox(tm) VectorDemo
 *  Copyright (C) 2009  Alec Bourque
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This program demonstrate mode 6, a 1-bit ramtiles mode that allows 256 tiles.
 */

#include <stdbool.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <uzebox.h>
#include <videoMode6/videoMode6.h>

#include "StringObjectsFonts.h"

#define MAX_OBJS 42
#define OBJ_LEN 12

#define OBJ_MIN_ENEMY     3
#define OBJ_MAX_ENEMY     9

#define OBJ_BULLET        0
#define OBJ_SHIP          1
#define OBJ_MIS_BLAST     2



#define OBJ_ROCK_1        3
#define OBJ_ROCK_2        4
#define OBJ_ROCK_3        5
#define OBJ_GROUND_FLAT   6
#define OBJ_GROUND_PEAK   7
#define OBJ_BOUDLER       8
#define OBJ_LANDING_PAD   9
#define OBJ_BROKEN_SHIP_1 10
#define OBJ_BROKEN_SHIP_2 11
#define OBJ_BROKEN_SHIP_3 12
#define OBJ_BROKEN_SHIP_4 13
#define OBJ_PARTICLE      14
#define OBJ_SAFE_SHIP     15
#define OBJ_SIGHT         16
#define OBJ_MC_GROUND_1   17



// C/ASM shared globals for screen/objects

unsigned char ObjectStore[MAX_OBJS][OBJ_LEN] __attribute__ ((section (".objectstore")));
unsigned char ramTiles[256*8]  __attribute__ ((section (".ramtiles")));
unsigned char vram[32*28] __attribute__ ((section (".vram")));
unsigned char trigtable[64] __attribute__ ((section (".trigtable")));

extern u8 sync_flags;
extern volatile u8 sync_pulse;
extern volatile uint8_t nextFreeRamTile;
extern volatile uint8_t renderCount;
extern volatile uint8_t ClearVramFlags;
extern volatile uint8_t StatusTiles[11];

// Screen management routines


extern void DefaultCallback(void);
extern void mycallback(void);
extern uint8_t GetRamTileFastC(uint8_t X, uint8_t Y);
extern uint8_t OutStringXYProgmemFastC(uint8_t X, uint8_t Y, const char *StringAddress);
extern void OutCharXYFastC(uint8_t X, uint8_t Y, uint8_t CharNum);
extern void OutHex8XYFastC(uint8_t X, uint8_t Y, uint8_t HexNum);
extern void OutDec8XYFastC(uint8_t X, uint8_t Y, uint8_t HexNum);
extern void SetPixelFastC(uint8_t X, uint8_t Y);
extern void ClearBufferLastLine(void);
extern void ClearBufferLastLineTileOnly(void);
extern void Mode7PutCharFastC(uint8_t x, uint8_t y, uint8_t CharNo);
extern void DrawPolarObjectFastC(uint8_t x, uint8_t y, uint8_t CharNo, uint8_t Theta, uint8_t scale);
extern void LineMode7FastC(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1);
extern void bresh_line_asm(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1);
extern uint16_t AddBCD(uint16_t Accum, uint8_t Add);
void Mode7PutChar(uint8_t x, uint8_t y, uint8_t CharNo);
void UpdateStatusLine(uint8_t Line, char FirstChar, uint16_t Value);
void ClearStatus(void);

// Geometry routines

extern int8_t SinMulFastC(int8_t angle, uint8_t distance);
extern int8_t CosMulFastC(int8_t angle, uint8_t distance);
extern int8_t SinFastC(int8_t angle);
extern int8_t CosFastC(int8_t angle);
PointStruct PolarToPoint(PointStruct P1, PolarPointStruct PP1);
uint8_t CheckCollision(ObjectDescStruct *Ob1, ObjectDescStruct *Ob2);
uint8_t PointInTriangle(PointStruct P1, PointStruct P2, PointStruct P3, PointStruct PX);

// Object management routines

void ClearObjectStore(void);
uint8_t GetFreeObject(void);
uint8_t NewShip(void);
uint8_t NewBullet(uint8_t X, uint8_t Y, int8_t dX, int8_t dY, uint8_t L);
uint8_t NewParticle(uint8_t X, uint8_t Y, int8_t dX, int8_t dY);
uint8_t NewRock(uint8_t Type, uint8_t X, uint8_t Y, int8_t dX, int8_t dY, int8_t dT, uint8_t Scale);
uint8_t NewRockRand();
uint8_t NewObject(uint8_t Type, uint8_t X, uint8_t Y, uint8_t T, int8_t dX, int8_t dY, int8_t dT, uint8_t Scale, uint8_t Life);

// Game logic routines

void PlayLevelAsteroid(uint8_t NumRocks);
void PlayLevelGyrus(uint8_t NumRocks, uint8_t Freq);
void PlayLevelGravity(uint8_t Map);
void PlayLevelMC(uint8_t NumRocks, uint8_t Freq);



void ScrollText(void);
void ProcessInput(void);
void MoveObjects(void);
void DrawObjects(void);
void CollisionDetection(void);
uint8_t CheckSafe(uint8_t X0, uint8_t Y0, uint8_t X1, uint8_t Y1);

void ProcessInputGyrus(void);
void MoveObjectsGyrus(void);
void ConvertShipCoordToGyrus(void);

void MoveObjectsGravity(void);

void ProcessInputMC(void);
void MoveObjectsMC(void);


void ProcessInputDraw(void);


// C Globals

ObjectDescStruct *Ship;
uint16_t Score;
uint8_t  Lives;
uint8_t  Wave;
uint8_t  RockCount;
int16_t	 dXSub;
int16_t	 dYSub;
uint8_t  SpawnX;
uint8_t  SpawnY;




uint8_t CheckSafe(uint8_t X0, uint8_t Y0, uint8_t X1, uint8_t Y1) {

uint8_t i;

ObjectDescStruct *Current;


for(i = 0; i < MAX_OBJS; i++) {

	Current = (ObjectDescStruct*)&ObjectStore[i];

	if ((Current->Type > 0x01) && (Current->Type < 0x05)) {

		if ((Current->X_Hi > X0) &&
			(Current->X_Hi < X1) &&
			(Current->Y_Hi > Y0) &&
			(Current->Y_Hi < Y1)) {

			return(1);
		}

	}

}
return(0);
}


void ConvertShipCoordToGyrus(void) {

Ship->X_Hi = SinMulFastC((Ship->T_Hi+128),200) + 128;
Ship->Y_Hi = CosMulFastC((Ship->T_Hi+128),200) + 112;
}

void PlayLevelAsteroid(uint8_t NumRocks){

uint8_t i;

ClearStatus();

UpdateStatusLine(0, 'P', Score);
UpdateStatusLine(1, 'L', Lives);
UpdateStatusLine(2, 'W', Wave);

SetRenderingParameters((21), (224));

ClearObjectStore();


Ship = NULL;
SpawnX = 128;
SpawnY = 128;

RockCount = 0;

for(i = 0; i < NumRocks; i++) {
	NewRockRand();
}

while((RockCount > 0) && (Lives > 0)) {

	ClearVramFlags = ClearFrameYes;

	while(renderCount != 0);			// GetVSync doesn't always work use my own counter
    ClearVsyncFlag();

	ClearBufferLastLine();
	ClearVramFlags = ClearFrameNo;

	DrawObjects();

	while(!GetVsyncFlag());
	ClearVsyncFlag();

	if (Ship == NULL) {
		Ship = ((ObjectDescStruct*)&ObjectStore[(int)NewShip()]);
		dXSub = 0;
		dYSub = 0;
	}
	if (Ship->Type == OBJ_SAFE_SHIP) {
		
		if (CheckSafe(85, 69, 171, 155) == 0) {
			Ship->Type = OBJ_SHIP;
		}
	}
	if (Ship->Type == OBJ_SHIP) ProcessInput();

	CollisionDetection();
	MoveObjects();
}
}


void PlayLevelGyrus(uint8_t NumRocks, uint8_t Freq){

uint8_t RandTimeToRock;
uint8_t RandRockType;

ClearStatus();

UpdateStatusLine(0, 'P', Score);
UpdateStatusLine(1, 'L', Lives);
UpdateStatusLine(2, 'W', Wave);

SetRenderingParameters((21), (224));

ClearObjectStore();

Ship = NULL;
SpawnX = 128;
SpawnY = 128;

RockCount = NumRocks;
RandTimeToRock = 0;
RandRockType = rand()%3;

while((RockCount > 0) && (Lives > 0)) {

	ClearVramFlags = ClearFrameYes;

	while(renderCount != 0);			// GetVSync doesn't always work use my own counter
    ClearVsyncFlag();

	ClearBufferLastLine();
	ClearVramFlags = ClearFrameNo;

	DrawObjects();

	while(!GetVsyncFlag());
	ClearVsyncFlag();

	if (Ship == NULL) {
		Ship = ((ObjectDescStruct*)&ObjectStore[(int)NewShip()]);
		dXSub = 0;
		dYSub = 0;
	}
	if (Ship->Type == OBJ_SAFE_SHIP) {
		
		if (CheckSafe(95, 130, 161, 224) == 0) {
			Ship->Type = OBJ_SHIP;
		}
	}
	if (Ship->Type == OBJ_SHIP) ProcessInputGyrus();

	if (RandTimeToRock == 0) {

		RandTimeToRock = rand()%255;

		NewRock(rand()%3+OBJ_MIN_ENEMY, 128, 112, SinMulFastC(RandTimeToRock, 40) , CosMulFastC(RandTimeToRock, 40) , rand()%0x80 - 0x40, 1);
		RockCount--;
			
		RandTimeToRock = rand()%Freq + 10;
	}

	RandTimeToRock--;

	ConvertShipCoordToGyrus();

	CollisionDetection();
	MoveObjectsGyrus();
}
}



void PlayLevelGravity(uint8_t Map) {

uint16_t i = 0;
uint8_t j = 0;

uint8_t Obj, X, Y, T, dX, dY, dT, Scale;

uint8_t PadX0, PadY, PadX1, Landed;

ClearStatus();

UpdateStatusLine(0, 'P', Score);
UpdateStatusLine(1, 'L', Lives);
UpdateStatusLine(2, 'W', Wave);

SetRenderingParameters((21), (224));

ClearObjectStore();

Ship = NULL;
RockCount = 3;

while(j < Map) {
	Obj   = pgm_read_byte(&GravLevels[i+0]);
	if (Obj == 255) j++;
	i = i + 8;
}

do {
	Obj   = pgm_read_byte(&GravLevels[i+0]);
	X     = pgm_read_byte(&GravLevels[i+1]);
	Y     = pgm_read_byte(&GravLevels[i+2]);
	T     = pgm_read_byte(&GravLevels[i+3]);
	dX    = pgm_read_byte(&GravLevels[i+4]);
	dY    = pgm_read_byte(&GravLevels[i+5]);
	dT    = pgm_read_byte(&GravLevels[i+6]);
	Scale = pgm_read_byte(&GravLevels[i+7]);

	NewObject(Obj, X, Y, T, dX, dY, dT, Scale, 0);

	i = i + 8;
} while (Obj != 255);

SpawnX = X;
SpawnY = Y;
PadX0  = dX;
PadY   = dY;
PadX1  = dT;
Landed = 0;


while((RockCount > 0) && (Lives > 0)) {

	ClearVramFlags = ClearFrameYes;

	while(renderCount != 0);			// GetVSync doesn't always work use my own counter
    ClearVsyncFlag();

	ClearBufferLastLine();
	ClearVramFlags = ClearFrameNo;

	DrawObjects();

	while(!GetVsyncFlag());
	ClearVsyncFlag();

	if (Ship == NULL) {
		Ship = ((ObjectDescStruct*)&ObjectStore[(int)NewShip()]);
		dXSub = 0;
		dYSub = 0;
	}
	if (Ship->Type == OBJ_SAFE_SHIP) {
		
		if (CheckSafe(95, 130, 161, 224) == 0) {
			Ship->Type = 1;
		}
	}

	if (Ship->Type == OBJ_SHIP) ProcessInput();

//	ProcessInputDraw();


	if ((Ship->X_Hi > PadX0) && (Ship->X_Hi < PadX1) && (Ship->Y_Hi == PadY) && (dYSub < 2000)) {

		dXSub = 0;
		Ship->dX = 0;

		if (dYSub > 0) dYSub = 0;
		if (Ship->dY > 0 ) Ship->dY = 0;

		if (Landed == 0) {

			Score = AddBCD(Score, 10);
			UpdateStatusLine(0, 'P', Score);

			Landed = 1;
		}


	} else {

		dYSub = dYSub + 100;
		if(dYSub > 0x3600) dYSub = 0x3600;
		Ship->dY = dYSub>>8;
	}

	CollisionDetection();
	MoveObjectsGravity();

	if (Ship->Y_Hi < 7) {

		if (Landed == 0) {
			Ship->Y_Hi = 7;
		} else {
			RockCount = 0;
		}

	}

}
}


void PlayLevelMC(uint8_t NumRocks, uint8_t Speed) {

uint8_t RandTimeToRock;
uint8_t RandRockType;

ClearStatus();

UpdateStatusLine(0, 'P', Score);
UpdateStatusLine(1, 'L', Lives);
UpdateStatusLine(2, 'W', Wave);

SetRenderingParameters((21), (224));

ClearObjectStore();

SpawnX = 128;
SpawnY = 128;
dXSub = 0;
dYSub = 0;

Ship = ((ObjectDescStruct*)&ObjectStore[(int)NewShip()]);
Ship->Type = OBJ_SIGHT;
Ship->X_Hi = SpawnX;
Ship->Y_Hi = SpawnY;

NewObject(OBJ_MC_GROUND_1,  20, 222,  10, 0, 0, 0, 255, 0);
NewObject(OBJ_MC_GROUND_1,  50, 222, 245, 0, 0, 0, 255, 0);
NewObject(OBJ_MC_GROUND_1,  90, 222,  10, 0, 0, 0, 255, 0);
NewObject(OBJ_MC_GROUND_1, 115, 218, 245, 0, 0, 0, 150, 0);

NewObject(OBJ_MC_GROUND_1, 135, 218,  10, 0, 0, 0, 150, 0);
NewObject(OBJ_MC_GROUND_1, 160, 222, 245, 0, 0, 0, 255, 0);
NewObject(OBJ_MC_GROUND_1, 200, 222,  10, 0, 0, 0, 255, 0);
NewObject(OBJ_MC_GROUND_1, 230, 222, 245, 0, 0, 0, 255, 0);

NewObject(OBJ_MC_GROUND_1, 125, 205, 205, 0, 0, 0,  80, 0);
NewObject(OBJ_MC_GROUND_1, 125, 205,  50, 0, 0, 0,  80, 0);

RockCount = NumRocks;
RandTimeToRock = 0;
RandRockType = rand()%3;


while((RockCount > 0) && (Lives > 0)) {

	ClearVramFlags = ClearFrameYes;

	while(renderCount != 0);			// GetVSync doesn't always work use my own counter
    ClearVsyncFlag();

	ClearBufferLastLine();
	ClearVramFlags = ClearFrameNo;

	DrawObjects();

	while(!GetVsyncFlag());
	ClearVsyncFlag();

	ProcessInputMC();

	if (RandTimeToRock == 0) {

		NewRock(4, (rand()%185 + 35), 0, rand()%8-4 , (5*Speed) , rand()%0x80 - 0x40, 50);
		RockCount--;
			
		RandTimeToRock = rand()%60 + 10;
	}

	RandTimeToRock--;


	CollisionDetection();
	MoveObjectsMC();


}
}

void PlayMessage(uint8_t MsgNum) {

uint16_t i = 0;
uint8_t j = 0;
uint8_t c;
uint8_t x = 0;
uint8_t y = 0;
uint8_t p0 = 0;
uint8_t p1 = 0;

while (j != MsgNum) {
	c = pgm_read_byte(&LevelText[i]);
	if (c == 0xFF) j++;
	i++;
}

ClearStatus();
SetRenderingParameters((21), (224));
ClearObjectStore();
ClearVramFlags = ClearFrameYes;

while(renderCount != 0);			// GetVSync doesn't always work use my own counter

ClearVsyncFlag();

ClearBufferLastLine();
ClearVramFlags = ClearFrameNo;

do {

	c = pgm_read_byte(&LevelText[i]);
	i++;

	switch (c) {
		case 0xFF : break;
		case 0xFE : {
			x = pgm_read_byte(&LevelText[i]);
			i++;
			y = pgm_read_byte(&LevelText[i]);
			i++;
			break;
		}
		case 0xFD : {
			p0 = pgm_read_byte(&LevelText[i]);
			i++;
			break;
		}
		case 0xFC : {
			p1 = pgm_read_byte(&LevelText[i]);
			i++;
			break;
		}
		default : {
			OutCharXYFastC(x,y,c);
			p0 = p1;
			x++;
		}
	}

	renderCount = 0;
	while (renderCount < p0);

} while ((ReadJoypad(0) == 0) && (c != 0xFF));

while (ReadJoypad(0) == 0);
while (ReadJoypad(0) != 0);

}



void GameWon(void){
PlayMessage(4);
}

void GameOver(void){

ClearStatus();
SetRenderingParameters((21), (224));
ClearObjectStore();
ClearVramFlags = ClearFrameYes;

while(renderCount != 0);			// GetVSync doesn't always work use my own counter

ClearVsyncFlag();

ClearBufferLastLine();
ClearVramFlags = ClearFrameNo;

OutStringXYProgmemFastC(12,13,strptr(str_GameOver));

while (ReadJoypad(0) == 0);

}

void PlayGame(void) {

uint8_t Wave_Temp;
uint8_t FirstPlay = 0;

Score = 0;
Lives = 3;
Wave  = 1;


//PlayLevelAsteroid(Wave);
//PlayLevelGyrus(50, 80);
//PlayLevelGravity(2);
//PlayLevelMC(15,60);


	do {

		Wave_Temp = Wave - 1;

		switch (Wave_Temp & 0b00000011) {

			case 0 : {
				if (FirstPlay == 0) PlayMessage(0);
				PlayLevelAsteroid((Wave_Temp >> 2) + 1);
				break;
			}
			case 1 : {
				if (FirstPlay == 0) PlayMessage(1);
				PlayLevelGyrus((Wave_Temp << 2) + 10, 50 - (Wave_Temp << 2));
				break;
			}
			case 2 : {
				if (FirstPlay == 0) PlayMessage(2);
				PlayLevelGravity(Wave_Temp >> 2);
				break;
			}
			case 3 : {
				if (FirstPlay == 0) PlayMessage(3);
				FirstPlay = 1;
				PlayLevelMC((Wave_Temp << 2) + 5, ((Wave_Temp >> 2)+1));
				break;
			}
			default : break;
		}

		Wave++;
	} while ((Lives > 0) && (Wave <= 12));


if ((Wave == 13) && (Lives > 0)) {
	GameWon();
} else {
	GameOver();
}

}

int main(){

srand(123);

SetHsyncCallback(&DefaultCallback);

ScrollText();

while(1){
	PlayGame();
}

}
 












void ProcessInputMC(void) {

static uint8_t BulletCountDown = 0;

uint16_t buttons;

buttons = ReadJoypad(0);

if(buttons & BTN_RIGHT) {
	Ship->X_Hi = Ship->X_Hi + 3;
}

if(buttons & BTN_LEFT)  {
	Ship->X_Hi = Ship->X_Hi - 3;
}

if(buttons & BTN_UP) {
	Ship->Y_Hi = Ship->Y_Hi - 3;
	if (Ship->Y_Hi < 30) Ship->Y_Hi = 30;
}

if(buttons & BTN_DOWN) {
	Ship->Y_Hi = Ship->Y_Hi + 3;
	if (Ship->Y_Hi > 160) Ship->Y_Hi = 160;
}


if((buttons & BTN_A) && (BulletCountDown == 0))    {
	NewBullet(125, 200, ((-(125 - Ship->X_Hi))>>1), ((-(200 - Ship->Y_Hi))>>1), 32);
	BulletCountDown = 10;
}

if(BulletCountDown != 0) {
	BulletCountDown--;
}



}



void ProcessInputDraw(void) {

uint16_t buttons;

buttons = ReadJoypad(0);

if(buttons & BTN_RIGHT) {
	if(buttons & BTN_A) {
		Ship->T_Hi = Ship->T_Hi - 1;
	} else {
		Ship->X_Hi = Ship->X_Hi + 1;
	}
}

if(buttons & BTN_LEFT)  {
	if(buttons & BTN_A) {
		Ship->T_Hi = Ship->T_Hi + 1;
	} else {
		Ship->X_Hi = Ship->X_Hi - 1;
	}
}

if(buttons & BTN_UP) {
	if(buttons & BTN_A) {
		Ship->Scale = Ship->Scale + 5;
	} else {
		Ship->Y_Hi = Ship->Y_Hi - 1;
	}
}

if(buttons & BTN_DOWN) {
	if(buttons & BTN_A) {
		Ship->Scale = Ship->Scale - 5;
	} else {
		Ship->Y_Hi = Ship->Y_Hi + 1;
	}
}

if(buttons & BTN_B) {
	Ship->Type = Ship->Type + 1;
	if(Ship->Type > 9) Ship->Type = 1;
}

/*
OutDec8XYFastC(20,11,Ship->Type);
OutDec8XYFastC(20,12,Ship->X_Hi);
OutDec8XYFastC(20,13,Ship->Y_Hi);
OutDec8XYFastC(20,14,Ship->T_Hi);
OutDec8XYFastC(20,15,Ship->Scale);
*/

}





















void ProcessInputGyrus(void) {

static uint8_t BulletCountDown = 0;

uint8_t RandThrustAngle;

uint16_t buttons;

buttons = ReadJoypad(0);

if(buttons & BTN_RIGHT) {
	Ship->T_Hi = Ship->T_Hi + 3;
}

if(buttons & BTN_LEFT)  {
	Ship->T_Hi = Ship->T_Hi - 3;
}

RandThrustAngle = 148 - rand()%40;
NewParticle(Ship->X_Hi + SinMulFastC((Ship->T_Hi + 128),10),
            Ship->Y_Hi + CosMulFastC((Ship->T_Hi + 128),10),
			Ship->dX   + SinMulFastC((Ship->T_Hi + RandThrustAngle),64),
			Ship->dY   + CosMulFastC((Ship->T_Hi + RandThrustAngle),64));

if((buttons & BTN_A) && (BulletCountDown == 0))    {
	NewBullet(Ship->X_Hi, Ship->Y_Hi, Ship->dX + SinMulFastC(Ship->T_Hi,64), Ship->dY + CosMulFastC(Ship->T_Hi,64), 40);
	BulletCountDown = 10;
}

if(BulletCountDown != 0) {
	BulletCountDown--;
}

}



void ProcessInput(void) {

static uint8_t BulletCountDown = 0;

uint8_t RandThrustAngle;

uint16_t buttons;

buttons = ReadJoypad(0);

if(buttons & BTN_RIGHT) {
	Ship->T_Hi = Ship->T_Hi - 3;
}

if(buttons & BTN_LEFT)  {
	Ship->T_Hi = Ship->T_Hi + 3;
}

if((buttons & BTN_UP) || (buttons & BTN_B))    {
	dXSub = dXSub + (SinFastC(Ship->T_Hi)<<2);
	dYSub = dYSub +  + (CosFastC(Ship->T_Hi)<<2);

	if(dXSub >  0x3600) dXSub =  0x3600;
	if(dXSub < -0x3600) dXSub = -0x3600;

	if(dYSub >  0x3600) dYSub =  0x3600;
	if(dYSub < -0x3600) dYSub = -0x3600;


	Ship->dX = dXSub>>8;
	Ship->dY = dYSub>>8;
	
	RandThrustAngle = 148 - rand()%40;
	NewParticle(Ship->X_Hi + SinMulFastC((Ship->T_Hi + 128),10),
	            Ship->Y_Hi + CosMulFastC((Ship->T_Hi + 128),10),
				Ship->dX   + SinMulFastC((Ship->T_Hi + RandThrustAngle),15),
				Ship->dY   + CosMulFastC((Ship->T_Hi + RandThrustAngle),15));
}

if((buttons & BTN_A) && (BulletCountDown == 0))    {
	NewBullet(Ship->X_Hi, Ship->Y_Hi, Ship->dX + SinMulFastC(Ship->T_Hi,64), Ship->dY + CosMulFastC(Ship->T_Hi,64), 80);
	BulletCountDown = 10;
}

if(BulletCountDown != 0) {
	BulletCountDown--;
}

}

void ClearStatus(void)
{
uint8_t i;

for (i = 0; i < 11; i++) {
    StatusTiles[i] = ' ';
}
}

void UpdateStatusLine(uint8_t Line, char FirstChar, uint16_t Value)
{

switch(Line) {
	case 0 : {
		StatusTiles[0]  = FirstChar;
		StatusTiles[1]  = ((Value>>12) & 0x000F) + '0';
		StatusTiles[2]  = ((Value>> 8) & 0x000F) + '0';
		StatusTiles[3]  = ((Value>> 4) & 0x000F) + '0';
		StatusTiles[4]  = (Value & 0x000F) + '0';
		break;		
	}
	case 1 : {
		StatusTiles[5]  = FirstChar;
		StatusTiles[6]  = ((Value>> 4) & 0x000F) + '0';
		StatusTiles[7]  = (Value & 0x000F) + '0';
		break;		
	}
	case 2 : {
		StatusTiles[8]  = FirstChar;
		StatusTiles[9]  = ((Value>> 4) & 0x000F) + '0';
		StatusTiles[10] = (Value & 0x000F) + '0';
		break;		
	}
	default : {
	}

}
}

void ScrollText()
{
uint8_t subcount = 0;
int16_t yoff_start = 225;
int16_t yoff_end;

uint8_t i, j;

ClearBuffer();

SetRenderingParameters((144+21), (224-144));
do {
	while(!GetVsyncFlag());

	ClearVramFlags = ClearFrameNo;
	ClearBufferLastLineTileOnly();

	yoff_end = yoff_start;

	for(i = 0; i < 20; i++) {

		for (j = 0; j < 12; j++) {
		
			if ((yoff_end >=0 ) & (yoff_end < 224)) {
				Mode7PutCharFastC((j*21), yoff_end, pgm_read_byte(&CrawlText[i][j]));
			}
		}

		yoff_end = yoff_end + 24;
	}

	if (subcount == 0) yoff_start--;
	subcount++;
	if (subcount > 1) subcount = 0;

	ClearVramFlags = ClrRamTileOlny | ClearFrameYes;  //11 = ram tile only.
    ClearVsyncFlag();

} while ((yoff_end > 0) && (0 == ReadJoypad(0)));

while (ReadJoypad(0) != 0);

ClearBuffer();

SetRenderingParameters((21), (224));
}

void ClearObjectStore(void)
{
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	Current->Type = 0xFF;
}
}

uint8_t GetFreeObject(void)
{
uint8_t i = 0;
uint8_t Particle = 0xFF;

ObjectDescStruct *Current;

do {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if (Current->Type == 0xFF) {
		return(i);
	}
	if (Current->Type == OBJ_PARTICLE) {
		Particle = i;
	}
	i++;
} while (i<MAX_OBJS);

return(Particle);
}

void DrawObjects(void) {
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if(Current->Type != 0xFF) {
		DrawPolarObjectFastC(Current->X_Hi,  Current->Y_Hi, Current->Type, Current->T_Hi, Current->Scale);
	}
}
}

void MoveObjectsGyrus(void) {
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if(Current->Type != 0xFF) {
		Current->X16 = Current->X16 + ((Current->dX)<<4);
		Current->Y16 = Current->Y16 + ((Current->dY)<<4);
		Current->T16 = Current->T16 + ((Current->dT)<<3);

		if((Current->Y_Hi > 223) && (Current->Y_Hi) <= 255) {
			if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) RockCount--; // Rock of screen destory it and no points.
			Current->Type = 0xFF;
		}
		if((Current->X_Hi < 10)  || (Current->X_Hi) >= 245) {
			if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) RockCount--;    // Rock of screen destory it and no points.
			Current->Type = 0xFF;
		}

		if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) {
			Current->Scale++;
		}

		if(Current->Life != 0xFF) {
			Current->Life--;
			if(Current->Life == 0) {

				if (Current->Type == OBJ_BULLET) {
					Current->Type = 0xFF;
				}

				if (Current->Type == OBJ_BROKEN_SHIP_1) {
					Ship = NULL;
					Current->Type = 0xFF;
					Lives--;
					UpdateStatusLine(1, 'L', Lives);
				}

				if (Current->Type == OBJ_SHIP) {
					Ship->Type = OBJ_BROKEN_SHIP_1;
					Ship->Life = 60;
					NewObject(OBJ_BROKEN_SHIP_2, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_3, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_4, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
				}

				if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) {

					Score = AddBCD(Score, 0x01);
					RockCount--;
					Current->Type = 0xFF;
					UpdateStatusLine(0, 'P', Score);
				}

				if ((Current->Type <= OBJ_PARTICLE) && (Current->Type >= OBJ_BROKEN_SHIP_2)) {
					Current->Type = 0xFF;
				}
			}
		} 
	}
}
}

void MoveObjectsMC(void) {
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if(Current->Type != 0xFF) {
		Current->X16 = Current->X16 + ((Current->dX)<<4);
		Current->Y16 = Current->Y16 + ((Current->dY)<<4);
		Current->T16 = Current->T16 + ((Current->dT)<<3);

		if((Current->Y_Hi > 223) && (Current->dY) > 0) {
			Current->Type = 0xFF;
			Lives--;
			UpdateStatusLine(1, 'L', Lives);
		}


		if(Current->Type == OBJ_MIS_BLAST) Current->Scale = Current->Scale + 5;

		if(Current->Life != 0xFF) {
			Current->Life--;
			if(Current->Life == 0) {

				if(Current->Type == OBJ_MIS_BLAST) {
					Current->Type = 0xFF;
				}

				if (Current->Type == OBJ_BULLET) {
					Current->Type = OBJ_MIS_BLAST;
					Current->Scale = 1;
					Current->Life = 30;
					Current->dX = 0;
					Current->dY = 0;
				}

				if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) {
					Score = AddBCD(Score, 0x03);
					Current->Type = 0xFF;
					UpdateStatusLine(0, 'P', Score);
					RockCount--;
				}
	
				if ((Current->Type <= OBJ_PARTICLE) && (Current->Type >= OBJ_BROKEN_SHIP_2)) {
					Current->Type = 0xFF;
				}


			}
		} 
	}
}
}


void MoveObjects(void) {
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if(Current->Type != 0xFF) {
		Current->X16 = Current->X16 + ((Current->dX)<<4);
		Current->Y16 = Current->Y16 + ((Current->dY)<<4);
		Current->T16 = Current->T16 + ((Current->dT)<<3);

		if((Current->Y_Hi > 223) && (Current->dY) > 0) Current->Y_Hi = 0;
		if((Current->Y_Hi > 223) && (Current->dY) < 0) Current->Y_Hi = 223;

		if(Current->Life != 0xFF) {
			Current->Life--;
			if(Current->Life == 0) {

				if (Current->Type == OBJ_BULLET) {
					Current->Type = 0xFF;
				}

				if (Current->Type == OBJ_BROKEN_SHIP_1) {
					Ship = NULL;
					Current->Type = 0xFF;
					Lives--;
					UpdateStatusLine(1, 'L', Lives);
				}

				if (Current->Type == OBJ_SHIP) {
					Ship->Type = OBJ_BROKEN_SHIP_1;
					Ship->Life = 60;
					NewObject(OBJ_BROKEN_SHIP_2, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_3, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_4, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
				}

				if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) {
					switch (Current->Scale) {
						case 80 : {
							Score = AddBCD(Score, 0x01);
							NewRock(Current->Type, Current->X_Hi, Current->Y_Hi, rand()%0x30 - 0x15, rand()%0x30 - 0x15, rand()%0x80 - 0x40, 55);
							NewRock(Current->Type, Current->X_Hi, Current->Y_Hi, rand()%0x30 - 0x15, rand()%0x30 - 0x15, rand()%0x80 - 0x40, 55);
							RockCount--;
							break;
						}
						case 55 : {
							Score = AddBCD(Score, 0x02);
							NewRock(Current->Type, Current->X_Hi, Current->Y_Hi, rand()%0x30 - 0x15, rand()%0x30 - 0x15, rand()%0x80 - 0x40, 30);
							NewRock(Current->Type, Current->X_Hi, Current->Y_Hi, rand()%0x30 - 0x15, rand()%0x30 - 0x15, rand()%0x80 - 0x40, 30);
							NewRock(Current->Type, Current->X_Hi, Current->Y_Hi, rand()%0x30 - 0x15, rand()%0x30 - 0x15, rand()%0x80 - 0x40, 30);
							RockCount--;
							break;
						}
						case 30 : {
							RockCount--;
							Score = AddBCD(Score, 0x03);
							break;
						}
						default : break;					
					}					
					Current->Type = 0xFF;
					UpdateStatusLine(0, 'P', Score);
				}
	
				if ((Current->Type <= OBJ_PARTICLE) && (Current->Type >= OBJ_BROKEN_SHIP_2)) {
					Current->Type = 0xFF;
				}

			}
		} 
	}
}
}

void MoveObjectsGravity(void) {
uint8_t i;
ObjectDescStruct *Current;

for(i = 0; i < MAX_OBJS; i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];
	if(Current->Type != 0xFF) {
		Current->X16 = Current->X16 + ((Current->dX)<<4);
		Current->Y16 = Current->Y16 + ((Current->dY)<<4);
		Current->T16 = Current->T16 + ((Current->dT)<<3);

		if((Current->Y_Hi > 223) && (Current->dY) > 0) Current->Y_Hi = 0;
		if((Current->Y_Hi > 223) && (Current->dY) < 0) Current->Y_Hi = 223;

		if(Current->Life != 0xFF) {
			Current->Life--;
			if(Current->Life == 0) {

				if (Current->Type == OBJ_BULLET) {
					Current->Type = 0xFF;
				}

				if (Current->Type == OBJ_BROKEN_SHIP_1) {
					Ship = NULL;
					Current->Type = 0xFF;
					Lives--;
					UpdateStatusLine(1, 'L', Lives);
				}

				if (Current->Type == OBJ_SHIP) {
					Ship->Type = OBJ_BROKEN_SHIP_1;
					Ship->Life = 60;
					NewObject(OBJ_BROKEN_SHIP_2, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_3, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
					NewObject(OBJ_BROKEN_SHIP_4, Ship->X_Hi, Ship->Y_Hi, Ship->T_Hi, Ship->dX + rand()%4-2, Ship->dY + rand()%4-2, Ship->dT + rand()%80-40, 255, 60);
				}

				if ((Current->Type <= OBJ_PARTICLE) && (Current->Type >= OBJ_BROKEN_SHIP_2)) {
					Current->Type = 0xFF;
				}

			}
		} 
	}
}
}



uint8_t CheckCollision(ObjectDescStruct *Ob1, ObjectDescStruct *Ob2) {  //Ob1 is Ship/bullet  Ob2 is rock

//  Idea here to save both time and improve accuracy of small rock collision detection.
//
//	1, Make bounding box check dynamic (25 pixels is too large a comparison for small rock and anything
//			So make bound-check a variable that is dependant on rock size (1,2 or 3) and bullet-or-ship.
//
//  2, instead of doing "Dist*scale" for every point in a rock and every point in a bullet/ship
//       do Dist/(other)Scale for the points in the bullet/ship only and leave rock "unscaled"
//       This will save avg of 12 MULs per rock.  The DIV is expensive, but will do a lookup table of 1/MUL = DIV
//       The lookup table will be slow and take 3 clocks from flash but only has to be done once per deep-collision
//       detection.  Just need to check if it is accurate enough.




uint8_t i,j;

uint8_t AbsDistCheck;

uint8_t Dist1Read;
uint8_t Dist2Read;

PolarPointStruct PP1, PP2, PPX;
PointStruct      P1, P2, P3, PX;
PointStruct      Ob1Point;

AbsDistCheck = (Ob2->Scale >> 3) + 5;						// This is fairly dodgy "scale to bounding box" maths that "just works"
if (Ob1->Type == OBJ_SHIP) AbsDistCheck = AbsDistCheck + 5;			// in this case.  It needs some more thought to be general purpose.
if (Ob1->Type == OBJ_MIS_BLAST) AbsDistCheck = AbsDistCheck + 30;			// in this case.  It needs some more thought to be general purpose.

if((abs(Ob1->X_Hi - Ob2->X_Hi) < AbsDistCheck) &&
   (abs(Ob1->Y_Hi - Ob2->Y_Hi) < AbsDistCheck)) { // Possible collision (By bounding Box)

	Ob1Point.X = Ob1->X_Hi;
	Ob1Point.Y = Ob1->Y_Hi;

	P1.X = Ob2->X_Hi;
	P1.Y = Ob2->Y_Hi;

	i = 0;

	Dist1Read = pgm_read_byte(&PolarObjects[Ob2->Type].Points[i].Dist);
	PP2.Dist  = (Dist1Read * Ob2->Scale)>>8;
	PP2.Theta = pgm_read_byte(&PolarObjects[Ob2->Type].Points[i].Theta) + Ob2->T_Hi;
	
	while (Dist1Read != 0xFF) {

		PP1 = PP2;

		i++;
		Dist1Read = pgm_read_byte(&PolarObjects[Ob2->Type].Points[i].Dist);
		PP2.Dist  = (Dist1Read * Ob2->Scale)>>8;;
		PP2.Theta = pgm_read_byte(&PolarObjects[Ob2->Type].Points[i].Theta) + Ob2->T_Hi;

		P2 = PolarToPoint(P1, PP1);
		P3 = PolarToPoint(P1, PP2);

		if(Dist1Read == 0xFF) break;

		j = 0;

		Dist2Read = pgm_read_byte(&PolarObjects[Ob1->Type].Points[j].Dist);
		PPX.Dist  = (Dist2Read * Ob1->Scale)>>8;
		PPX.Theta = pgm_read_byte(&PolarObjects[Ob1->Type].Points[j].Theta) + Ob1->T_Hi;
	
		while (Dist2Read != 0xFF) {

			PX = PolarToPoint(Ob1Point, PPX);

			if(PointInTriangle(P1, P2, P3, PX) == 1) {
				return(1);
			}

			j++;
			Dist2Read = pgm_read_byte(&PolarObjects[Ob1->Type].Points[j].Dist);
			PPX.Dist  = (Dist2Read * Ob1->Scale)>>8;
			PPX.Theta = pgm_read_byte(&PolarObjects[Ob1->Type].Points[j].Theta) + Ob1->T_Hi;
		}
	}
}

return(0);

}

void CollisionDetection(void) {

uint8_t i, j;
ObjectDescStruct *Current;
ObjectDescStruct *Compare;

for(i = 0; i < (MAX_OBJS - 1); i++) {
	Current = (ObjectDescStruct*)&ObjectStore[i];

	if (Current->Type < OBJ_MIN_ENEMY) { 										// If Bullet or Ship

		for(j = (i + 1); j < MAX_OBJS; j++) {

			Compare = (ObjectDescStruct*)&ObjectStore[j];

			if ((Compare->Type > (OBJ_MIN_ENEMY - 1)) && (Compare->Type < (OBJ_MAX_ENEMY + 1))) {	// Then compare with Rocks
				if (CheckCollision(Current, Compare) != 0) {
					
					if (Compare->Type != OBJ_MIS_BLAST) Compare->Life = 1;
					if (Current->Type != OBJ_MIS_BLAST) Current->Life = 1;


				} 
			}
		}
	} else if ((Current->Type > (OBJ_MIN_ENEMY - 1)) && (Current->Type < (OBJ_MAX_ENEMY + 1))) { 		// If Rocks

		for(j = (i + 1); j < MAX_OBJS; j++) {

			Compare = (ObjectDescStruct*)&ObjectStore[j];

			if (Compare->Type < OBJ_MIN_ENEMY) {									// Then compare with ship/bullet
				if (CheckCollision(Compare, Current) != 0) {
					
					if (Compare->Type != OBJ_MIS_BLAST) Compare->Life = 1;
					if (Current->Type != OBJ_MIS_BLAST) Current->Life = 1;


				} 
			}
		}
	}

}
}



PointStruct PolarToPoint(PointStruct P1, PolarPointStruct PP1) {

PointStruct PX;

PX.X = P1.X + SinMulFastC(PP1.Theta, PP1.Dist);
PX.Y = P1.Y + CosMulFastC(PP1.Theta, PP1.Dist);

return(PX);
}

uint8_t PointInTriangle(PointStruct P1, PointStruct P2, PointStruct P3, PointStruct PX) {

int16_t PlaneAB;
int16_t PlaneBC;
int16_t PlaneCA;


// For testing collision detection
/*
SetPixelFastC(P1.X, P1.Y);
SetPixelFastC(P2.X, P2.Y);
SetPixelFastC(P3.X, P3.Y);
*/


PlaneAB = ((uint8_t)P1.X - (uint8_t)PX.X) * ((uint8_t)P2.Y - (uint8_t)PX.Y) - ((uint8_t)P2.X - (uint8_t)PX.X) * ((uint8_t)P1.Y - (uint8_t)PX.Y);
if(PlaneAB > 0) return(0);

PlaneBC = ((uint8_t)P2.X - (uint8_t)PX.X) * ((uint8_t)P3.Y - (uint8_t)PX.Y) - ((uint8_t)P3.X - (uint8_t)PX.X) * ((uint8_t)P2.Y - (uint8_t)PX.Y);
if(PlaneBC > 0) return(0);

PlaneCA = ((uint8_t)P3.X - (uint8_t)PX.X) * ((uint8_t)P1.Y - (uint8_t)PX.Y) - ((uint8_t)P1.X - (uint8_t)PX.X) * ((uint8_t)P3.Y - (uint8_t)PX.Y);
if(PlaneCA > 0) return(0);

return(1);
}


uint8_t NewRockRand(){

ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =  rand()%0x3 + OBJ_MIN_ENEMY;
	NewObj->X_Hi  =  rand()%0xFF;
	NewObj->Y_Hi  =  rand()%0xFF;
	NewObj->T_Hi  =  rand()%0xFF;
	NewObj->dX    =  rand()%0x30 - 0x15;
	NewObj->dY    =  rand()%0x30 - 0x15;
	NewObj->dT    =  rand()%0x80 - 0x40;
	NewObj->Scale =  80;
	NewObj->Life  =   0;

	RockCount++;

}
return(NewObjNum);
}

uint8_t NewParticle(uint8_t X, uint8_t Y, int8_t dX, int8_t dY){

ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =  OBJ_PARTICLE;
	NewObj->X_Hi  =     X;
	NewObj->Y_Hi  =     Y;
	NewObj->T_Hi  =     0;
	NewObj->dX    =    dX;
	NewObj->dY    =    dY;
	NewObj->dT    =     0;
	NewObj->Scale =     1;
	NewObj->Life  =    15;

}
return(NewObjNum);
}

uint8_t NewRock(uint8_t Type, uint8_t X, uint8_t Y, int8_t dX, int8_t dY, int8_t dT, uint8_t Scale) {

ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =  Type;
	NewObj->X_Hi  =     X;
	NewObj->Y_Hi  =     Y;
	NewObj->T_Hi  =  rand()%0xFF;
	NewObj->dX    =    dX;
	NewObj->dY    =    dY;
	NewObj->dT    =    dT;
	NewObj->Scale = Scale;
	NewObj->Life  =     0;

	RockCount++;

}
return(NewObjNum);
}

uint8_t NewBullet(uint8_t X, uint8_t Y, int8_t dX, int8_t dY, uint8_t L)
{
ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =   OBJ_BULLET;
	NewObj->X_Hi  =   X;
	NewObj->Y_Hi  =   Y;
	NewObj->T_Hi  =   0;
	NewObj->dX    =  dX;
	NewObj->dY    =  dY;
	NewObj->dT    =   0;
	NewObj->Scale =   1;
	NewObj->Life  =   L;

}
return(NewObjNum);
}


uint8_t NewShip(void) {

ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =   OBJ_SAFE_SHIP;
	NewObj->X_Hi  = SpawnX;
	NewObj->Y_Hi  = SpawnY;
	NewObj->T_Hi  = 128;
	NewObj->dX    =   0;
	NewObj->dY    =   0;
	NewObj->dT    =   0;
	NewObj->Scale = 255;
	NewObj->Life  =   0;

}
return(NewObjNum);
}

uint8_t NewObject(uint8_t Type, uint8_t X, uint8_t Y, uint8_t T, int8_t dX, int8_t dY, int8_t dT, uint8_t Scale, uint8_t Life) {

ObjectDescStruct *NewObj;
uint8_t NewObjNum;

NewObjNum = GetFreeObject();

if(NewObjNum != 0xFF) {

	NewObj = ((ObjectDescStruct*)&ObjectStore[NewObjNum]);

	NewObj->Type  =  Type;
	NewObj->X_Hi  =     X;
	NewObj->Y_Hi  =     Y;
	NewObj->T_Hi  =     T;
	NewObj->dX    =    dX;
	NewObj->dY    =    dY;
	NewObj->dT    =    dT;
	NewObj->Scale = Scale;
	NewObj->Life  =  Life;

}
return(NewObjNum);
}
